@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2004-2013 OpenWorks LLP
+   Copyright (C) 2004-2017 OpenWorks LLP
       info@open-works.net
 
    This program is free software; you can redistribute it and/or
@@ -21,9 +21,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
    The GNU General Public License is contained in the file COPYING.
 
@@ -41,13 +39,15 @@
 #include "libvex.h"
 
 /* Top level optimiser entry point.  Returns a new BB.  Operates
-   under the control of the global "vex_control" struct. */
+   under the control of the global "vex_control" struct and of the
+   supplied |pxControl| argument. */
 extern 
-IRSB* do_iropt_BB(
+IRSB* do_iropt_BB (
          IRSB* bb,
          IRExpr* (*specHelper) (const HChar*, IRExpr**, IRStmt**, Int),
-         Bool (*preciseMemExnsFn)(Int,Int),
-         Addr64 guest_addr,
+         Bool (*preciseMemExnsFn)(Int,Int,VexRegisterUpdates),
+         VexRegisterUpdates pxControl,
+         Addr    guest_addr,
          VexArch guest_arch
       );
 
@@ -62,9 +62,19 @@ void do_deadcode_BB ( IRSB* bb );
 /* The tree-builder.  Make (approximately) maximal safe trees.  bb is
    destructively modified.  Returns (unrelatedly, but useful later on)
    the guest address of the highest addressed byte from any insn in
-   this block, or Addr64_MAX if unknown (can that ever happen?) */
+   this block, or Addr_MAX if unknown (can that ever happen?) */
 extern
-Addr64 ado_treebuild_BB ( IRSB* bb, Bool (*preciseMemExnsFn)(Int,Int) );
+Addr ado_treebuild_BB (
+        IRSB* bb,
+        Bool (*preciseMemExnsFn)(Int,Int,VexRegisterUpdates),
+        VexRegisterUpdates pxControl
+     );
+
+IRSB* do_minimal_initial_iropt_BB(
+         IRSB* bb0
+      );
+void concatenate_irsbs ( IRSB* dst, IRSB* src );
+void deltaIRStmt ( IRStmt* st, Int delta );
 
 #endif /* ndef __VEX_IR_OPT_H */
 
