@@ -1536,7 +1536,6 @@ static ULong DEBUG_SnarfLinetab(
    Int                k;
    const UInt         * lt_ptr;
    Int                nfile;
-   Int                nseg;
    union any_size     pnt;
    union any_size     pnt2;
    const struct startend * start;
@@ -1560,10 +1559,8 @@ static ULong DEBUG_SnarfLinetab(
    /*
     * Now count up the number of segments in the file.
     */
-   nseg = 0;
    for (i = 0; i < nfile; i++) {
       pnt2.c = (const HChar *)linetab + filetab[i];
-      nseg += *pnt2.s;
    }
 
    this_seg = 0;
