@@ -190,7 +190,7 @@ Int valgrind_stopped_by_syscall (void)
    return sysno_to_report;
 }
 
-Bool valgrind_stopped_before_syscall()
+Bool valgrind_stopped_before_syscall(void)
 {
    vg_assert (sysno_to_report >= 0);
    return before_syscall;
@@ -867,6 +867,8 @@ void valgrind_initialize_target(void)
    mips64_init_architecture(&the_low_target);
 #elif defined(VGA_nanomips)
    nanomips_init_architecture(&the_low_target);
+#elif defined(VGA_riscv64)
+   riscv64_init_architecture(&the_low_target);
 #else
    #error "architecture missing in target.c valgrind_initialize_target"
 #endif
